@@ -1,4 +1,4 @@
-# $Id: tables.py 7662 2013-05-16 20:38:43Z milde $
+# $Id: tables.py 7958 2016-07-28 21:52:14Z milde $
 # Authors: David Goodger <goodger@python.org>; David Priest
 # Copyright: This module has been placed in the public domain.
 
@@ -20,6 +20,10 @@ from docutils.parsers.rst import Directive
 from docutils.parsers.rst import directives
 
 
+def align(argument):
+    return directives.choice(argument, ('left', 'center', 'right'))
+
+
 class Table(Directive):
 
     """
@@ -29,7 +33,10 @@ class Table(Directive):
     optional_arguments = 1
     final_argument_whitespace = True
     option_spec = {'class': directives.class_option,
-                   'name': directives.unchanged}
+                   'name': directives.unchanged,
+                   'align': align,
+                   'widths': directives.value_or(('auto', 'grid'),
+                                                 directives.positive_int_list)}
     has_content = True
 
     def make_title(self):
@@ -85,15 +92,19 @@ class Table(Directive):
                     self.block_text, self.block_text), line=self.lineno)
                 raise SystemMessagePropagation(error)
 
+    @property
+    def widths(self):
+        return self.options.get('widths', '')
+
     def get_column_widths(self, max_cols):
-        if 'widths' in self.options:
-            col_widths = self.options['widths']
-            if len(col_widths) != max_cols:
+        if type(self.widths) == list:
+            if len(self.widths) != max_cols:
                 error = self.state_machine.reporter.error(
                     '"%s" widths do not match the number of columns in table '
                     '(%s).' % (self.name, max_cols), nodes.literal_block(
                     self.block_text, self.block_text), line=self.lineno)
                 raise SystemMessagePropagation(error)
+            col_widths = self.widths
         elif max_cols:
             col_widths = [100 // max_cols] * max_cols
         else:
@@ -101,7 +112,13 @@ class Table(Directive):
                 'No table data detected in CSV file.', nodes.literal_block(
                 self.block_text, self.block_text), line=self.lineno)
             raise SystemMessagePropagation(error)
-        return col_widths
+        if self.widths == 'auto':
+            widths = 'auto'
+        elif self.widths: # "grid" or list of integers
+            widths = 'given'
+        else:
+            widths = self.widths
+        return widths, col_widths
 
     def extend_short_rows_with_empty_cells(self, columns, parts):
         for part in parts:
@@ -130,6 +147,21 @@ class RSTTable(Table):
             return [error]
         table_node = node[0]
         table_node['classes'] += self.options.get('class', [])
+        if 'align' in self.options:
+            table_node['align'] = self.options.get('align')
+        tgroup = table_node[0]
+        if type(self.widths) == list:
+            colspecs = [child for child in tgroup.children
+                        if child.tagname == 'colspec']
+            for colspec, col_width in zip(colspecs, self.widths):
+                colspec['colwidth'] = col_width
+        # @@@ the colwidths argument for <tgroup> is not part of the
+        # XML Exchange Table spec (https://www.oasis-open.org/specs/tm9901.htm)
+        # and hence violates the docutils.dtd.
+        if self.widths == 'auto':
+            table_node['classes'] += ['colwidths-auto']
+        elif self.widths: # "grid" or list of integers
+            table_node['classes'] += ['colwidths-given']
         self.add_name(table_node)
         if title:
             table_node.insert(0, title)
@@ -141,12 +173,14 @@ class CSVTable(Table):
     option_spec = {'header-rows': directives.nonnegative_int,
                    'stub-columns': directives.nonnegative_int,
                    'header': directives.unchanged,
-                   'widths': directives.positive_int_list,
+                   'widths': directives.value_or(('auto', ),
+                                                 directives.positive_int_list),
                    'file': directives.path,
                    'url': directives.uri,
                    'encoding': directives.encoding,
                    'class': directives.class_option,
                    'name': directives.unchanged,
+                   'align': align,
                    # field delimiter char
                    'delim': directives.single_char_or_whitespace_or_unicode,
                    # treat whitespace after delimiter as significant
@@ -170,14 +204,14 @@ class CSVTable(Table):
 
         def __init__(self, options):
             if 'delim' in options:
-                self.delimiter = str(options['delim'])
+                self.delimiter = CSVTable.encode_for_csv(options['delim'])
             if 'keepspace' in options:
                 self.skipinitialspace = False
             if 'quote' in options:
-                self.quotechar = str(options['quote'])
+                self.quotechar = CSVTable.encode_for_csv(options['quote'])
             if 'escape' in options:
                 self.doublequote = False
-                self.escapechar = str(options['escape'])
+                self.escapechar = CSVTable.encode_for_csv(options['escape'])
             csv.Dialect.__init__(self)
 
 
@@ -219,21 +253,26 @@ class CSVTable(Table):
             self.check_table_dimensions(rows, header_rows, stub_columns)
             table_head.extend(rows[:header_rows])
             table_body = rows[header_rows:]
-            col_widths = self.get_column_widths(max_cols)
+            widths, col_widths = self.get_column_widths(max_cols)
             self.extend_short_rows_with_empty_cells(max_cols,
                                                     (table_head, table_body))
         except SystemMessagePropagation, detail:
             return [detail.args[0]]
         except csv.Error, detail:
+            message = str(detail)
+            if sys.version_info < (3,) and '1-character string' in message:
+                message += '\nwith Python 2.x this must be an ASCII character.'
             error = self.state_machine.reporter.error(
                 'Error with CSV data in "%s" directive:\n%s'
-                % (self.name, detail), nodes.literal_block(
+                % (self.name, message), nodes.literal_block(
                 self.block_text, self.block_text), line=self.lineno)
             return [error]
         table = (col_widths, table_head, table_body)
         table_node = self.state.build_table(table, self.content_offset,
-                                            stub_columns)
+                                            stub_columns, widths=widths)
         table_node['classes'] += self.options.get('class', [])
+        if 'align' in self.options:
+            table_node['align'] = self.options.get('align')
         self.add_name(table_node)
         if title:
             table_node.insert(0, title)
@@ -353,13 +392,15 @@ class ListTable(Table):
     Implement tables whose data is encoded as a uniform two-level bullet list.
     For further ideas, see
     http://docutils.sf.net/docs/dev/rst/alternatives.html#list-driven-tables
-    """ 
+    """
 
     option_spec = {'header-rows': directives.nonnegative_int,
                    'stub-columns': directives.nonnegative_int,
-                   'widths': directives.positive_int_list,
+                   'widths': directives.value_or(('auto', ),
+                                                 directives.positive_int_list),
                    'class': directives.class_option,
-                   'name': directives.unchanged}
+                   'name': directives.unchanged,
+                   'align': align}
 
     def run(self):
         if not self.content:
@@ -372,7 +413,7 @@ class ListTable(Table):
         node = nodes.Element()          # anonymous container for parsing
         self.state.nested_parse(self.content, self.content_offset, node)
         try:
-            num_cols, col_widths = self.check_list_content(node)
+            num_cols, widths, col_widths = self.check_list_content(node)
             table_data = [[item.children for item in row_list[0]]
                           for row_list in node[0]]
             header_rows = self.options.get('header-rows', 0)
@@ -380,8 +421,10 @@ class ListTable(Table):
             self.check_table_dimensions(table_data, header_rows, stub_columns)
         except SystemMessagePropagation, detail:
             return [detail.args[0]]
-        table_node = self.build_table_from_list(table_data, col_widths,
+        table_node = self.build_table_from_list(table_data, widths, col_widths,
                                                 header_rows, stub_columns)
+        if 'align' in self.options:
+            table_node['align'] = self.options.get('align')
         table_node['classes'] += self.options.get('class', [])
         self.add_name(table_node)
         if title:
@@ -424,15 +467,20 @@ class ListTable(Table):
                     raise SystemMessagePropagation(error)
             else:
                 num_cols = len(item[0])
-        col_widths = self.get_column_widths(num_cols)
-        return num_cols, col_widths
+        widths, col_widths = self.get_column_widths(num_cols)
+        return num_cols, widths, col_widths
 
-    def build_table_from_list(self, table_data, col_widths, header_rows, stub_columns):
+    def build_table_from_list(self, table_data, widths, col_widths, header_rows,
+                              stub_columns):
         table = nodes.table()
+        if widths:
+            table['classes'] += ['colwidths-%s' % widths]
         tgroup = nodes.tgroup(cols=len(col_widths))
         table += tgroup
         for col_width in col_widths:
-            colspec = nodes.colspec(colwidth=col_width)
+            colspec = nodes.colspec()
+            if col_width is not None:
+                colspec.attributes['colwidth'] = col_width
             if stub_columns:
                 colspec.attributes['stub'] = 1
                 stub_columns -= 1
