@@ -1,7 +1,7 @@
 #! /usr/bin/env python
-# -*- coding: utf8 -*-
+# -*- coding: utf-8 -*-
 
-# $Id: test_inline_markup.py 7243 2011-12-05 19:35:32Z milde $
+# $Id: test_inline_markup.py 7942 2016-05-25 12:48:25Z milde $
 # Author: David Goodger <goodger@python.org>
 # Copyright: This module has been placed in the public domain.
 
@@ -78,12 +78,13 @@ across lines*
 [r"""some punctuation is allowed around inline markup, e.g.
 /*emphasis*/, -*emphasis*-, and :*emphasis*: (delimiters),
 (*emphasis*), [*emphasis*], <*emphasis*>, {*emphasis*} (open/close pairs)
+*emphasis*., *emphasis*,, *emphasis*!, and *emphasis*\ (closing delimiters),
 
 but not
-)*emphasis*(, ]*emphasis*[, >*emphasis*>, }*emphasis*{ (close/open pairs)
+)*emphasis*(, ]*emphasis*[, >*emphasis*>, }*emphasis*{ (close/open pairs),
 (*), [*], '*' or '"*"' ("quoted" start-string),
 x*2* or 2*x* (alphanumeric char before),
-\*args or * (escaped, whitespace behind start-string)
+\*args or * (escaped, whitespace behind start-string),
 or *the\* *stars\* *inside* (escaped, whitespace before end-string).
 
 However, '*args' will trigger a warning and may be problematic.
@@ -117,12 +118,24 @@ what about *this**?
         <emphasis>
             emphasis
         } (open/close pairs)
+        <emphasis>
+            emphasis
+        ., \n\
+        <emphasis>
+            emphasis
+        ,, \n\
+        <emphasis>
+            emphasis
+        !, and \n\
+        <emphasis>
+            emphasis
+        (closing delimiters),
     <paragraph>
         but not
-        )*emphasis*(, ]*emphasis*[, >*emphasis*>, }*emphasis*{ (close/open pairs)
+        )*emphasis*(, ]*emphasis*[, >*emphasis*>, }*emphasis*{ (close/open pairs),
         (*), [*], '*' or '"*"' ("quoted" start-string),
         x*2* or 2*x* (alphanumeric char before),
-        *args or * (escaped, whitespace behind start-string)
+        *args or * (escaped, whitespace behind start-string),
         or \n\
         <emphasis>
             the* *stars* *inside
@@ -132,7 +145,7 @@ what about *this**?
         <problematic ids="id2" refid="id1">
             *
         args' will trigger a warning and may be problematic.
-    <system_message backrefs="id2" ids="id1" level="2" line="12" source="test data" type="WARNING">
+    <system_message backrefs="id2" ids="id1" level="2" line="13" source="test data" type="WARNING">
         <paragraph>
             Inline emphasis start-string without end-string.
     <paragraph>
@@ -1037,6 +1050,128 @@ Relative URIs' reference text can be omitted:
         <reference name="anonymous" refuri="anonymous">
             anonymous
 """],
+[r"""
+Escape trailing low-line char in URIs:
+
+`<reference\_>`_
+
+`<anonymous\_>`__
+""",
+"""\
+<document source="test data">
+    <paragraph>
+        Escape trailing low-line char in URIs:
+    <paragraph>
+        <reference name="reference_" refuri="reference_">
+            reference_
+        <target ids="reference" names="reference_" refuri="reference_">
+    <paragraph>
+        <reference name="anonymous_" refuri="anonymous_">
+            anonymous_
+"""],
+["""\
+Escape other char in URIs:
+
+`<reference\:1>`_
+
+`<anonymous\\call>`__
+""",
+"""\
+<document source="test data">
+    <paragraph>
+        Escape other char in URIs:
+    <paragraph>
+        <reference name="reference:1" refuri="reference:1">
+            reference:1
+        <target ids="reference-1" names="reference:1" refuri="reference:1">
+    <paragraph>
+        <reference name="anonymouscall" refuri="anonymouscall">
+            anonymouscall
+"""],
+]
+
+totest['embedded_aliases'] = [
+["""\
+`phrase reference <alias_>`_
+""",
+"""\
+<document source="test data">
+    <paragraph>
+        <reference name="phrase reference" refname="alias">
+            phrase reference
+        <target names="phrase\ reference" refname="alias">
+"""],
+["""\
+`anonymous reference <alias_>`__
+""",
+"""\
+<document source="test data">
+    <paragraph>
+        <reference name="anonymous reference" refname="alias">
+            anonymous reference
+"""],
+["""\
+`embedded alias on next line
+<alias_>`__
+""",
+"""\
+<document source="test data">
+    <paragraph>
+        <reference name="embedded alias on next line" refname="alias">
+            embedded alias on next line
+"""],
+["""\
+`embedded alias across lines <alias
+phrase_>`__
+""",
+"""\
+<document source="test data">
+    <paragraph>
+        <reference name="embedded alias across lines" refname="alias phrase">
+            embedded alias across lines
+"""],
+["""\
+`embedded alias with whitespace <alias 
+long  phrase_>`__
+""",
+"""\
+<document source="test data">
+    <paragraph>
+        <reference name="embedded alias with whitespace" refname="alias long phrase">
+            embedded alias with whitespace
+"""],
+[r"""
+`no embedded alias (whitespace inside bracket) < alias_ >`__
+
+`no embedded alias (no preceding whitespace)<alias_>`__
+""",
+"""\
+<document source="test data">
+    <paragraph>
+        <reference anonymous="1" name="no embedded alias (whitespace inside bracket) < alias_ >">
+            no embedded alias (whitespace inside bracket) < alias_ >
+    <paragraph>
+        <reference anonymous="1" name="no embedded alias (no preceding whitespace)<alias_>">
+            no embedded alias (no preceding whitespace)<alias_>
+"""],
+[r"""
+`anonymous reference <alias\ with\\ escaped \:characters_>`__
+""",
+"""\
+<document source="test data">
+    <paragraph>
+        <reference name="anonymous reference" refname="aliaswith\ escaped :characters">
+            anonymous reference
+"""],
+[r"""
+`anonymous reference <alias\ with\\ escaped \:characters_>`__
+""",
+"""\
+<document source="test data">
+    <paragraph>
+        <reference name="anonymous reference" refname="aliaswith\ escaped :characters">
+            anonymous reference
+"""],
 ]
 
 totest['inline_targets'] = [
